;;;; japi.scm

(declare
  ;(no-bound-checks)
  (hide vector->s32vector)
  (foreign-declare "#include \"japi.h\""))

(require 'srfi-4)

(define (vector->s32vector v)
  (let* ([n (vector-length v)]
	 [s (make-s32vector n)] )
    (do ([i 0 (fx+ i 1)])
	((fx>= i n) s)
      (s32vector-set! s i (##sys#slot v i)) ) ) )

(define-foreign-type char* c-string)
(define-foreign-type int* nonnull-s32vector vector->s32vector)

(eval-when (compile)

  (require 'srfi-13)

  (define (schemify name)
    (string-translate* (->string name) '(("j_" . "j:") ("_" . "-"))) )

  (define-macro (ci name val)
    (let* ([str (schemify (string-downcase (->string name)))]
	   [sname (string-translate* str '(("j:" . "j/")))] )
      `(begin
	 (define ,(string->symbol sname) ,val) ) ) )

  (define-macro (cdef rtype name . args)
    `(define ,(string->symbol (schemify name))
       (foreign-lambda ,rtype ,name ,@args) ) )

  (define-macro (void name . args) `(cdef void ,name ,@args))
  (define-macro (int name . args) `(cdef int ,name ,@args))
  (define-macro (integer name . args) `(cdef integer ,name ,@args))
  (define-macro (char* name . args) `(cdef char* ,name ,@args))
  (define-macro (c-string* name . args) `(cdef c-string* ,name ,@args))
  (define-macro (bool name . args) `(cdef bool ,name ,@args))

)


(ci J_LEFT                                  0)
(ci J_CENTER                                1)
(ci J_RIGHT                                 2)
(ci J_TOP                                   3)
(ci J_BOTTOM                                4)
(ci J_TOPLEFT                               5)
(ci J_TOPRIGHT                              6)
(ci J_BOTTOMLEFT                            7)
(ci J_BOTTOMRIGHT                           8)

(ci J_DEFAULT_CURSOR                        0)
(ci J_CROSSHAIR_CURSOR                      1)
(ci J_TEXT_CURSOR                           2)
(ci J_WAIT_CURSOR                           3)
(ci J_SW_RESIZE_CURSOR                      4)
(ci J_SE_RESIZE_CURSOR                      5)
(ci J_NW_RESIZE_CURSOR                      6)
(ci J_NE_RESIZE_CURSOR                      7)
(ci J_N_RESIZE_CURSOR                       8)
(ci J_S_RESIZE_CURSOR                       9)
(ci J_W_RESIZE_CURSOR                       10)
(ci J_E_RESIZE_CURSOR                       11)
(ci J_HAND_CURSOR                           12)
(ci J_MOVE_CURSOR                           13)

(ci J_HORIZONTAL                            0)
(ci J_VERTICAL                              1)

(ci J_PLAIN                                 0)
(ci J_BOLD                                  1)
(ci J_ITALIC                                2)
(ci J_COURIER                               1)
(ci J_HELVETIA                              2)
(ci J_TIMES                                 3)
(ci J_DIALOGIN                              4)
(ci J_DIALOGOUT                             5)

(ci J_BLACK                                 0)
(ci J_WHITE                                 1)
(ci J_RED                                   2)
(ci J_GREEN                                 3)
(ci J_BLUE                                  4)
(ci J_CYAN                                  5)
(ci J_MAGENTA                               6)
(ci J_YELLOW                                7)
(ci J_ORANGE                                8)
(ci J_GREEN_YELLOW                          9)
(ci J_GREEN_CYAN                            10)
(ci J_BLUE_CYAN                             11)
(ci J_BLUE_MAGENTA                          12)
(ci J_RED_MAGENTA                           13)
(ci J_DARK_GRAY                             14)
(ci J_LIGHT_GRAY                            15)
(ci J_GRAY                                  16)

(ci J_NONE                                  0)
(ci J_LINEDOWN                              1)
(ci J_LINEUP                                2)
(ci J_AREADOWN                              3)
(ci J_AREAUP                                4)

(ci J_MOVED                                 0)
(ci J_DRAGGED                               1)
(ci J_PRESSED                               2)
(ci J_RELEASED                              3)
(ci J_ENTERED                               4)
(ci J_EXITED                                5)
(ci J_DOUBLECLICK                           6)

(ci J_RESIZED                               1)
(ci J_HIDDEN                                2)
(ci J_SHOWN                                 3)

(ci J_ACTIVATED                             0)
(ci J_DEACTIVATED                           1)
(ci J_OPENED                                2)
(ci J_CLOSED                                3)
(ci J_ICONIFIED                             4)
(ci J_DEICONIFIED                           5)
(ci J_CLOSING                               6)

(ci J_GIF                                   0)
(ci J_JPG                                   1)
(ci J_PPM                                   2)
(ci J_BMP                                   3)

(ci J_ROUND                                 0)
(ci J_RECT                                  1)

(define j/randmax                               2147483647.)

(bool   j_start);
(int   j_connect char*  );
(void  j_setport int  );
(void  j_setdebug int  );
(int   j_frame char*  );
(int   j_button int char*  );
(int   j_graphicbutton int char*  );
(int   j_checkbox int char*  );
(int   j_label int char*  );
(int   j_graphiclabel int char*  );
(int   j_canvas int int int  );
(int   j_panel int  );
(int   j_borderpanel int int  );
(int   j_radiogroup int  );
(int   j_radiobutton int char*  );
(int   j_list int int  );
(int   j_choice int  );
(int   j_dialog int char*  );
(int   j_window int  );
(int   j_popupmenu int char*  );
(int   j_scrollpane int  );
(int   j_hscrollbar int  );
(int   j_vscrollbar int  );
(int   j_line int int int int  );
(int   j_printer int  );
(int   j_image int int  );
(c-string* j_filedialog int char* char* char*  );
(define j:filedialog
  (let ([j:filedialog j:filedialog])
    (lambda (f t d) (j:filedialog f t d #f)) ) )
(c-string* j_fileselect int char* char* char*  );
(define j:fileselect
  (let ([j:fileselect j:fileselect])
    (lambda (f t d) (j:fileselect f t d #f)) ) )
(int   j_messagebox int char* char*  );
(int   j_alertbox int char* char* char*  );
(int   j_choicebox2 int char* char* char* char*  );
(int   j_choicebox3 int char* char* char* char* char*  );
(int   j_progressbar int int  );
(int   j_led int int int  );
(int   j_sevensegment int int  );
(int   j_meter int char*  );
(void  j_additem int char*  );
(int   j_textfield int int  );
(int   j_textarea int int int  );
(int   j_menubar int  );
(int   j_menu int char*  );
(int   j_helpmenu int char*  );
(int   j_menuitem int char*  );
(int   j_checkmenuitem int char*  );
(void  j_pack int  );
(void  j_print int  );
(void  j_playsoundfile char*  );
(void  j_play int  );
(int   j_sound char*  );
(void  j_setfont int int int int  );
(void  j_setfontname int int  );
(void  j_setfontsize int int  );
(void  j_setfontstyle int int  );
(void  j_seperator int  );
(void  j_disable int  );
(void  j_enable int  );
(bool   j_getstate int  );
(int   j_getrows int  );
(int   j_getcolumns int  );
(int   j_getselect int  );
(bool   j_isselect int int  );
(bool   j_isvisible int  );
(bool   j_isparent int int  );
(bool   j_isresizable int  );
(void  j_select int int  );
(void  j_deselect int int  );
(void  j_multiplemode int bool  );
(void  j_insert int int char*  );
(void  j_remove int int  );
(void  j_removeitem int char*  );
(void  j_removeall int  );
(void  j_setstate int bool  );
(void  j_setrows int int  );
(void  j_setcolumns int int  );
(void  j_seticon int int  );
(void  j_setimage int int  );
(void  j_setvalue int int  );
(void  j_setradiogroup int int  );
(void  j_setunitinc int int  );
(void  j_setblockinc int int  );
(void  j_setmin int int  );
(void  j_setmax int int  );
(void  j_setdanger int int  );
(void  j_setslidesize int int  );
(void  j_setcursor int int  );
(void  j_setresizable int bool  );
(int   j_getlength int  );
(int   j_getvalue int  );
(int   j_getdanger int  );
(int   j_getscreenheight );
(int   j_getscreenwidth );
(int   j_getheight int  );
(int   j_getwidth int  );
(int   j_getinsets int int  );
(int   j_getlayoutid int  );
(int   j_getinheight int  );
(int   j_getinwidth int  );
(c-string* j_gettext int char*  );
(define j:gettext
  (let ([j:gettext j:gettext])
    (lambda (o) (j:gettext o #f)) ) )
(c-string* j_getitem int int char*);
(define j:getitem
  (let ([j:getitem j:getitem])
    (lambda (o i) (j:getitem o i #f)) ) )
(int   j_getitemcount int  );
(void  j_delete int int int  );
(void  j_replacetext int char* int int  );
(void  j_appendtext int char*  );
(void  j_inserttext int char* int  );
(void  j_settext int char*  );
(void  j_selectall int  );
(void  j_selecttext int int int  );
(int   j_getselstart int  );
(int   j_getselend int  );
(c-string* j_getseltext int char*);
(define j:getseltext
  (let ([j:getseltext j:getseltext])
    (lambda (o) (j:getseltext o #f)) ) )
(int   j_getcurpos int  );
(void  j_setcurpos int int  );
(void  j_setechochar int char  );
(void  j_seteditable int bool  );
(void  j_setshortcut int char  );
(void  j_quit );
(void  j_kill );
(void  j_setsize int int int  );
(int   j_getaction );
(int   j_nextaction );
(void  j_show int  );
(void  j_showpopup int int int  );
(void  j_add int int  );
(void  j_release int  );
(void  j_releaseall int  );
(void  j_hide int  );
(void  j_dispose int  );
(void  j_setpos int int int  );
(int   j_getviewportheight int  );
(int   j_getviewportwidth int  );
(int   j_getxpos int  );
(int   j_getypos int  );
;(void  j_getpos int int* int*  );
(int   j_getparentid int  );
(void  j_setfocus int  );
(bool   j_hasfocus int  );
(int   j_getstringwidth int char*  );
(int   j_getfontheight int  );
(int   j_getfontascent int  );
(int   j_keylistener int  );
(int   j_getkeycode int  );
(int   j_getkeychar int  );
(int   j_mouselistener int int  );
(int   j_getmousex int  );
(int   j_getmousey int  );
;(void  j_getmousepos int int* int*  );
(int   j_getmousebutton int  );
(int   j_focuslistener int  );
(int   j_componentlistener int int  );
(int   j_windowlistener int int  );
(void  j_setflowlayout int int  );
(void  j_setborderlayout int  );
(void  j_setgridlayout int int int  );
(void  j_setfixlayout int  );
(void  j_setnolayout int  );
(void  j_setborderpos int int  );
(void  j_sethgap int int  );
(void  j_setvgap int int  );
(void  j_setinsets int int int int int  );
(void  j_setalign int int  );
(void  j_setflowfill int bool  );
(void  j_translate int int int  );
(void  j_cliprect int int int int int  );
(void  j_drawrect int int int int int  );
(void  j_fillrect int int int int int  );
(void  j_drawroundrect int int int int int int int  );
(void  j_fillroundrect int int int int int int int  );
(void  j_drawoval int int int int int  );
(void  j_filloval int int int int int  );
(void  j_drawcircle int int int int  );
(void  j_fillcircle int int int int  );
(void  j_drawarc int int int int int int int  );
(void  j_fillarc int int int int int int int  );
(void  j_drawline int int int int int  );
(void  j_drawpolyline int int int* int*  );
(void  j_drawpolygon int int int* int*  );
(void  j_fillpolygon int int int* int*  );
(void  j_drawpixel int int int  );
(void  j_drawstring int int int char*  );
(void  j_setxor int bool  );
(int   j_getimage int  );
(void  j_getimagesource int int int int int int* int* int*  );
(void  j_drawimagesource int int int int int int* int* int*  );
(int   j_getscaledimage int int int int int int int  );
(void  j_drawimage int int int int  );
(void  j_drawscaledimage int int int int int int int int int int  );
(void  j_setcolor int int int int  );
(void  j_setcolorbg int int int int  );
(void  j_setnamedcolor int int  );
(void  j_setnamedcolorbg int int  );
(int   j_loadimage char*  );
(int   j_saveimage int char* int  );
(void  j_sync   );
(void  j_beep  );
(integer   j_random  );
(void  j_sleep int  );
